Package: flowGraph
Type: Package
Title: Identifying differential cell populations in flow cytometry data
        accounting for marker frequency
Version: 1.21.0
Description: Identifies maximal differential cell populations in flow cytometry data taking into account dependencies between cell populations; flowGraph calculates and plots SpecEnr abundance scores given cell population cell counts.
Date: 2019-11-30
Authors@R: person("Alice", "Yue", email = "aya43@sfu.ca", role = c("aut", "cre"))
License: Artistic-2.0
VignetteBuilder: knitr
Depends: R (>= 4.1)
Imports: effsize, furrr, future, purrr, ggiraph, ggrepel, ggplot2,
        igraph, Matrix, matrixStats, stats, utils, visNetwork,
        htmlwidgets, grDevices, methods, stringr, stringi, Rdpack,
        data.table (>= 1.9.5), gridExtra,
Suggests: BiocStyle, dplyr, knitr, rmarkdown, testthat (>= 2.1.0)
biocViews: FlowCytometry, StatisticalMethod, ImmunoOncology, Software,
        CellBasedAssays, Visualization
BugReports: https://github.com/aya49/flowGraph/issues
URL: https://github.com/aya49/flowGraph
RoxygenNote: 7.1.1
RdMacros: Rdpack
LazyData: no
git_url: https://git.bioconductor.org/packages/flowGraph
git_branch: devel
git_last_commit: 04c5027
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: no
Packaged: 2026-05-18 22:04:26 UTC; biocbuild
Author: Alice Yue [aut, cre]
Maintainer: Alice Yue <aya43@sfu.ca>
Built: R 4.6.0; ; 2026-05-19 04:51:37 UTC; unix
