# Generated by roxygen2: do not edit by hand

export(cell_type_layers)
export(fg_add_feature)
export(fg_add_summary)
export(fg_clear_features)
export(fg_clear_summary)
export(fg_extract_phenotypes)
export(fg_extract_raw)
export(fg_extract_samples)
export(fg_feat_edge_prop)
export(fg_feat_edge_specenr)
export(fg_feat_mean_class)
export(fg_feat_node_prop)
export(fg_feat_node_specenr)
export(fg_get_feature)
export(fg_get_feature_desc)
export(fg_get_feature_means)
export(fg_get_graph)
export(fg_get_markers)
export(fg_get_meta)
export(fg_get_summary)
export(fg_get_summary_desc)
export(fg_get_summary_index)
export(fg_get_summary_tables)
export(fg_gsub_ids)
export(fg_gsub_markers)
export(fg_load)
export(fg_merge)
export(fg_merge_samples)
export(fg_plot)
export(fg_plot_box)
export(fg_plot_pVSdiff)
export(fg_plot_qq)
export(fg_replace_meta)
export(fg_rm_feature)
export(fg_rm_summary)
export(fg_save)
export(fg_save_plots)
export(fg_set_layout)
export(fg_summary)
export(flowGraph)
export(flowGraphSubset)
export(flowGraphSubset_summary_adjust)
export(flowGraphSubset_summary_pars)
export(get_phen_list)
export(get_phen_meta)
export(ggdf)
export(plot_gr)
export(set_layout_graph)
export(summary_table)
exportClasses(flowGraph)
exportMethods(show)
import(igraph)
importFrom(Matrix,Matrix)
importFrom(Rdpack,reprompt)
importFrom(data.table,".I")
importFrom(data.table,as.data.table)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(data.table,setattr)
importFrom(effsize,cohen.d)
importFrom(furrr,future_map)
importFrom(future,multisession)
importFrom(future,plan)
importFrom(future,sequential)
importFrom(ggiraph,geom_boxplot_interactive)
importFrom(ggiraph,geom_point_interactive)
importFrom(ggiraph,geom_segment_interactive)
importFrom(ggiraph,girafe)
importFrom(ggplot2,aes_)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_dotplot)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_dodge)
importFrom(ggplot2,scale_colour_gradientn)
importFrom(ggplot2,scale_fill_brewer)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,stat_summary)
importFrom(ggplot2,theme)
importFrom(ggrepel,geom_label_repel)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,boxplot.stats)
importFrom(gridExtra,arrangeGrob)
importFrom(htmlwidgets,saveWidget)
importFrom(igraph,V)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,layout.reingold.tilford)
importFrom(matrixStats,rowMaxs)
importFrom(matrixStats,rowMins)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setMethod)
importFrom(methods,show)
importFrom(purrr,compact)
importFrom(purrr,map)
importFrom(purrr,map_chr)
importFrom(purrr,map_dbl)
importFrom(purrr,map_dfr)
importFrom(purrr,map_int)
importFrom(purrr,map_lgl)
importFrom(stats,chisq.test)
importFrom(stats,ks.test)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,t.test)
importFrom(stats,var.test)
importFrom(stats,wilcox.test)
importFrom(stringi,stri_rand_strings)
importFrom(stringr,str_count)
importFrom(stringr,str_extract)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_pad)
importFrom(stringr,str_split)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(utils,write.csv)
importFrom(visNetwork,visEdges)
importFrom(visNetwork,visEvents)
importFrom(visNetwork,visHierarchicalLayout)
importFrom(visNetwork,visInteraction)
importFrom(visNetwork,visNetwork)
importFrom(visNetwork,visNodes)
importFrom(visNetwork,visOptions)
importFrom(visNetwork,visSave)
